@@ -8,14 +8,12 @@ See the ```PointND``` struct for basic usage
 
 use std::{
     ops::{
-        Add, Sub, Mul, Div,
         Index, IndexMut
     },
     slice::SliceIndex,
     convert::TryInto,
 };
 
-
 /**
 
 The whole _point_ of the crate (get it?)
@@ -31,8 +29,7 @@ No matter how a PointND is constructed, the second generic arg must be filled wi
 If a point of zero dimensions is constructed, it will panic
 
 ```
-use point_nd::PointND;
-
+# use point_nd::PointND;
 // Creates a 2D point from values of a given vector or array
 let vec: Vec<i32> = vec![0, 1];
 let p: PointND<_, 2> = PointND::from(&vec);
@@ -54,8 +51,7 @@ let p = PointND::<_, 2>::from(&vec);
 It is recommended to use the convenience getters if the dimensions of the point are from ```1..=4```
 
 ```
-use point_nd::PointND;
-
+# use point_nd::PointND;
 // A 2D point
 let arr: [i32; 2] = [0,1];
 let p: PointND<_, 2> = PointND::from(&arr);
@@ -77,8 +73,7 @@ assert_eq!(*y, arr[1]);
 Otherwise indexing or the ```get()``` method can be used
 
 ```
-use point_nd::PointND;
-
+# use point_nd::PointND;
 let arr: [i32; 2] = [0,1];
 let p: PointND<_, 2> = PointND::from(&arr);
 
@@ -99,23 +94,42 @@ assert_eq!(y, arr[1]);
 The number of dimensions can be retrieved using the ```dims()``` method (short for _dimensions_)
 
 ```
-use point_nd::PointND;
-
+# use point_nd::PointND;
 let p: PointND<i32, 2> = PointND::fill(10);
 assert_eq!(p.dims(), 2);
 ```
 
+## Iterating
+
+The ```PointND``` struct does not implement iterating directly. The internal values must be accessed as an array in order to loop over them
+
+```
+# use point_nd::PointND;
+let arr: [i32; 4] = [0,1,2,3];
+let p: PointND<_, 4> = PointND::from(&arr);
+
+// Use either one of:
+let values:  [i32; 4] = p.as_arr();
+# for _ in values.iter() {}
+let values: &[i32; 4] = p.values();
+# for _ in values.iter() {}
+let values:  Vec<i32> = p.as_vec();
+
+for (i, item) in values.into_iter().enumerate() {
+    assert_eq!(item, arr[i]);
+}
+```
  */
-#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+#[derive(Clone, PartialEq, Eq, Debug)]
 pub struct PointND<T, const N: usize>
     where T: Clone + Copy  {
     arr: [T; N],
 }
 
-// Constructors
 impl<T, const N: usize>  PointND<T, N>
     where T: Clone + Copy  {
 
+    // Constructors
     /**
      Returns a new ```PointND``` with values from the specified array or vector
 
@@ -142,13 +156,8 @@ impl<T, const N: usize>  PointND<T, N>
         PointND::<T, N>::from(&[value; N])
     }
 
-}
-
-// Standard Getters
-impl<T, const N: usize>  PointND<T, N>
-    where T: Clone + Copy  {
-
 
+    // Standard Getters
     /**
      Returns the number of dimensions of the point (a 2D point will return 2, a 3D point 3, _etc_)
      */
@@ -165,20 +174,38 @@ impl<T, const N: usize>  PointND<T, N>
         self.arr.get(dim)
     }
 
-}
 
-// Modifiers
-impl<T, const N: usize>  PointND<T, N>
-    where T: Clone + Copy  {
+    // Wholesale getters
+    /**
+     Returns a pointer to the array values stored by self
+     */
+    pub fn values(&self) -> &[T; N] {
+        &self.arr
+    }
 
+    /**
+     Returns an array of all the values contained by the point
+     */
+    pub fn as_arr(&self) -> [T; N] {
+        self.arr.clone()
+    }
+
+    /**
+     Returns a vector of all the values contained by the point
+     */
+    pub fn as_vec(&self) -> Vec<T> {
+        Vec::from(&self.arr[..])
+    }
+
+
+    // Modifiers
     /**
      Returns a new ```PointND``` from the values contained by self after applying the modifier function to them
 
      ### Examples
 
      ```
-     use point_nd::PointND;
-
+     # use point_nd::PointND;
      // Multiplies each item by 10
      let p = PointND::<i32, 3>::from(&[0, 1, 2]);
      let p = p.apply(|item| item * 10);
@@ -186,6 +213,7 @@ impl<T, const N: usize>  PointND<T, N>
      assert_eq!(p.as_arr(), [0, 10, 20]);
      ```
      */
+    // Did not call apply_dims() inside this to avoid the dimension checks it does
     pub fn apply<F>(self, modifier: F) -> Self
         where F: Fn(T) -> T {
 
@@ -207,8 +235,7 @@ impl<T, const N: usize>  PointND<T, N>
     ### Examples
 
      ```
-     use point_nd::PointND;
-
+     # use point_nd::PointND;
      // Multiplies items at indexes 1 and 2 by 2
      let p = PointND::<i32, 4>::from(&[0, 1, 2, 3]);
      let p = p.apply_dims(&[1, 2], |item| item * 2);
@@ -237,16 +264,15 @@ impl<T, const N: usize>  PointND<T, N>
      ### Examples
 
      ```
-     use point_nd::PointND;
-
+     # use point_nd::PointND;
      // Adds each item in the PointND with their respective items in the array
      let p = PointND::<i32, 3>::from(&[0, 1, 2]);
-     let p = p.apply_with([1, 2, 3], |a, b| a + b);
+     let p = p.apply_vals([1, 2, 3], |a, b| a + b);
 
      assert_eq!(p.as_arr(), [1, 3, 5]);
      ```
      */
-    pub fn apply_with<F>(self, values: [T; N], modifier: F) -> Self
+    pub fn apply_vals<F>(self, values: [T; N], modifier: F) -> Self
         where F: Fn(T, T) -> T {
 
         let mut vec = Vec::<T>::with_capacity(N);
@@ -257,31 +283,13 @@ impl<T, const N: usize>  PointND<T, N>
         PointND::<T, N>::from(&vec)
     }
 
-}
-
-// Wholesale Getters
-impl<T, const N: usize>  PointND<T, N>
-    where T: Clone + Copy  {
-
     /**
-     Returns a pointer to the array values stored by self
+     Returns a new ```PointND``` from the values contained by self and those of the point specified after applying the modifier to both
      */
-    pub fn values(&self) -> &[T; N] {
-        &self.arr
-    }
+    pub fn apply_with<F>(self, other: PointND<T, N>, modifier: F) -> Self
+        where F: Fn(T, T) -> T {
 
-    /**
-     Returns an array of all the values contained by the point
-     */
-    pub fn as_arr(&self) -> [T; N] {
-        self.arr.clone()
-    }
-
-    /**
-     Returns a vector of all the values contained by the point
-     */
-    pub fn as_vec(&self) -> Vec<T> {
-        Vec::from(&self.arr[..])
+        self.apply_vals(other.as_arr(), modifier)
     }
 
 }
@@ -332,84 +340,10 @@ impl<T> PointND<T, 4> where T: Clone + Copy  {
 
 }
 
-// Basic math operators
-impl<T, const N: usize> Add for PointND<T, N> where T: Add<Output = T> + Clone + Copy  {
-
-    type Output = Self;
-    fn add(self, rhs: Self) -> Self::Output {
-        if &self.dims() != &rhs.dims() { panic!("Tried to add two PointND's of unequal length"); }
-
-        let values_left= self.values();
-        let values_right = rhs.values();
-
-        let mut ret_values = Vec::<T>::with_capacity(N);
-        for i in 0..N {
-            ret_values.push(values_left[i].clone() + values_right[i].clone());
-        }
-
-        PointND::<T, N>::from(&ret_values)
-    }
-
-}
-impl<T, const N: usize> Sub for PointND<T, N> where T: Sub<Output = T> + Clone + Copy  {
-
-    type Output = Self;
-    fn sub(self, rhs: Self) -> Self::Output {
-        if &self.dims() != &rhs.dims() { panic!("Tried to add two PointND's of unequal length"); }
-
-        let values_left= self.values();
-        let values_right = rhs.values();
-
-        let mut ret_values = Vec::<T>::with_capacity(N);
-        for i in 0..N {
-            ret_values.push(values_left[i].clone() - values_right[i].clone());
-        }
-
-        PointND::<T, N>::from(&ret_values)
-    }
-
-}
-impl<T, const N: usize> Mul for PointND<T, N> where T: Mul<Output = T> + Clone + Copy  {
-
-    type Output = Self;
-    fn mul(self, rhs: Self) -> Self::Output {
-        if &self.dims() != &rhs.dims() { panic!("Tried to add two PointND's of unequal length"); }
-
-        let values_left= self.values();
-        let values_right = rhs.values();
-
-        let mut ret_values = Vec::<T>::with_capacity(N);
-        for i in 0..N {
-            ret_values.push(values_left[i].clone() * values_right[i].clone());
-        }
-
-        PointND::<T, N>::from(&ret_values)
-    }
-
-}
-impl<T, const N: usize> Div for PointND<T, N> where T: Div<Output = T> + Clone + Copy  {
-
-    type Output = Self;
-    fn div(self, rhs: Self) -> Self::Output {
-        if &self.dims() != &rhs.dims() { panic!("Tried to add two PointND's of unequal length"); }
-
-        let values_left= self.values();
-        let values_right = rhs.values();
-
-        let mut ret_values = Vec::<T>::with_capacity(N);
-        for i in 0..N {
-            ret_values.push(values_left[i].clone() / values_right[i].clone());
-        }
-
-        PointND::<T, N>::from(&ret_values)
-    }
-
-}
-
 // Indexing operators
 impl<I, T, const N: usize> Index<I> for PointND<T, N> where T: Clone + Copy, I: Sized + SliceIndex<[T], Output = T> {
     type Output = T;
-    fn index(&self, index: I) -> & Self::Output {
+    fn index(&self, index: I) -> &Self::Output {
         &self.arr[index]
     }
 }
@@ -419,7 +353,6 @@ impl<I, T, const N: usize> IndexMut<I> for PointND<T, N> where T: Clone + Copy,
     }
 }
 
-
 #[cfg(test)]
 mod tests {
 
@@ -446,23 +379,12 @@ mod tests {
             impl A {
                 pub fn new(x: i32) -> Self { A{ x } }
             }
-            impl Add for A {
-                type Output = Self;
-                fn add(self, rhs: Self) -> Self::Output {
-                    A::new(self.x + rhs.x)
-                }
-            }
 
             let p = PointND::<A, 3>::fill(A::new(0));
-            let p = p + PointND::from(&[
-                A::new(1),
-                A::new(2),
-                A::new(3)
-            ]);
 
-            assert_ne!(p.x(), p.y());
-            assert_ne!(p.x(), p.z());
-            assert_ne!(p.y(), p.z());
+            assert_eq!(p.x(), p.y());
+            assert_eq!(p.x(), p.z());
+            assert_eq!(p.y(), p.z());
         }
 
         #[test]
@@ -580,67 +502,6 @@ mod tests {
             p[1002] = new_val;
         }
 
-        #[test]
-        fn can_add_two() {
-            let vec = vec![0,1,2,3];
-            let p1 = PointND::<_, 4>::from(&vec);
-            let p2 = PointND::from(&vec);
-
-            let p3 = p1 + p2;
-            for (a, b) in p3.as_arr().into_iter().zip(vec){
-                assert_eq!(a, b + b);
-            }
-        }
-
-        #[test]
-        fn can_subtract() {
-            let vec = vec![0,1,2,3];
-            let p1 = PointND::<_, 4>::from(&vec);
-            let p2 = PointND::from(&vec);
-
-            let p3 = p1 - p2;
-            for (a, b) in p3.as_arr().into_iter().zip(vec){
-                assert_eq!(a, b - b);
-            }
-        }
-
-        #[test]
-        fn can_multiply() {
-            let vec = vec![0,1,2,3];
-            let p1 = PointND::<_, 4>::from(&vec);
-            let p2 = PointND::from(&vec);
-
-            let p3 = p1 * p2;
-            for (a, b) in p3.as_arr().into_iter().zip(vec){
-                assert_eq!(a, b * b);
-            }
-        }
-
-        #[test]
-        fn can_divide() {
-            let vec = vec![1,2,3,4];
-            let p1 = PointND::<_, 4>::from(&vec);
-            let p2 = PointND::from(&vec);
-
-            let p3 = p1 / p2;
-            for (a, b) in p3.as_arr().into_iter().zip(vec){
-                assert_eq!(a, b / b);
-            }
-        }
-
-        #[test]
-        #[should_panic]
-        fn cannot_divide_if_one_item_is_zero() {
-            let vec = vec![0, 1,2,3,4];
-            let p1 = PointND::<_, 5>::from(&vec);
-            let p2 = PointND::from(&vec);
-
-            let p3 = p1 / p2;
-            for (a, b) in p3.as_arr().into_iter().zip(vec){
-                assert_eq!(a, b / b);
-            }
-        }
-
         #[test]
         fn can_equal() {
             let vec = vec![1,2,3,4];
@@ -688,16 +549,28 @@ mod tests {
         }
 
         #[test]
-        fn apply_with_does_work() {
+        fn apply_vals_does_work() {
 
             let arr = [0, 1, 2];
             let apply_values = [10, 20, 30];
             let arr_to_be = [10, 21, 32];
 
-            let p = PointND::<_, 3>::from(&arr).apply_with(apply_values, |a, b| a + b);
+            let p = PointND::<_, 3>::from(&arr).apply_vals(apply_values, |a, b| a + b);
+            assert_eq!(p.as_arr(), arr_to_be);
+        }
+
+        #[test]
+        fn apply_with_works() {
+
+            let p1 = PointND::<i32, 3>::fill(1);
+            let p2 = PointND::<_, 3>::from(&[10, 3, 45]);
+
+            let arr_to_be = [-9, -2, -44];
+            
+            let p = p1.apply_with(p2, |a, b| a - b);
             assert_eq!(p.as_arr(), arr_to_be);
         }
 
     }
 
-}
\ No newline at end of file
+}
